PHIDATA_RUNTIME_ENV_VAR: str = "PHIDATA_RUNTIME"

SCRIPTS_DIR_ENV_VAR: str = "PHI_SCRIPTS_DIR"
STORAGE_DIR_ENV_VAR: str = "PHI_STORAGE_DIR"
META_DIR_ENV_VAR: str = "PHI_META_DIR"
PRODUCTS_DIR_ENV_VAR: str = "PHI_PRODUCTS_DIR"
NOTEBOOKS_DIR_ENV_VAR: str = "PHI_NOTEBOOKS_DIR"
WORKSPACE_CONFIG_DIR_ENV_VAR: str = "PHI_WORKSPACE_CONFIG_DIR"

AWS_REGION_ENV_VAR: str = "AWS_REGION"
AWS_PROFILE_ENV_VAR: str = "AWS_PROFILE"
AWS_CONFIG_FILE_ENV_VAR: str = "AWS_CONFIG_FILE"
AWS_SHARED_CREDENTIALS_FILE_ENV_VAR: str = "AWS_SHARED_CREDENTIALS_FILE"
